"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const client_1 = require("@apollo/client");
// Optimistic response generation utility method
//
function generateOptimisticResponseForMutation({ operationType, entityName, objects }) {
    const optimisticResponse = {
        __typename: "mutation_root",
        [`${operationType}_${entityName}`]: {
            affected_rows: objects.length,
            returning: objects.map(entity => {
                return { ...entity, __typename: entityName };
            }),
            __typename: `\${entityName}_mutation_response`
        }
    };
    return optimisticResponse;
}
exports.generateOptimisticResponseForMutation = generateOptimisticResponseForMutation;
/**
 *
 * @param param0
 */
function generateUpdateFunctionForMutation({ operationType, entityName, entityId }) {
    return (cache, result) => {
        cache.evict(client_1.defaultDataIdFromObject({ __typename: entityName, id: entityId }));
    };
}
exports.generateUpdateFunctionForMutation = generateUpdateFunctionForMutation;
function IS_JAVASCRIPT_SCALAR_EQUIVALENT(arg, includeNull = false) {
    var type = typeof arg;
    return (includeNull && arg === null) || (type != "object" && type != "function");
}
exports.IS_JAVASCRIPT_SCALAR_EQUIVALENT = IS_JAVASCRIPT_SCALAR_EQUIVALENT;
function IS_APOLLO_OBJECT(o) {
    return o && o.id && o.__typename;
}
exports.IS_APOLLO_OBJECT = IS_APOLLO_OBJECT;
function convertApolloObjectToRefObj(o) {
    if (!IS_APOLLO_OBJECT(o))
        throw new Error(`Provided object was not of type ApolloObject (with id & _typename properties): ${JSON.stringify(o)}`);
    return { id: o.id, __typename: o.__typeName };
}
exports.convertApolloObjectToRefObj = convertApolloObjectToRefObj;
function convertApolloObjectArrayToRefObj(o) {
    return o.map(arrayItem => convertApolloObjectToRefObj(o));
}
exports.convertApolloObjectArrayToRefObj = convertApolloObjectArrayToRefObj;
function convertInsertInputToShallowPartialFragment(insertInputObject) {
    const fragment = {};
    // Loop object and build up a fragment appropriate for a cache-add
    for (const [insertInputKey, insertInputValue] of Object.entries(insertInputObject)) {
        //Add scalar values
        if (IS_JAVASCRIPT_SCALAR_EQUIVALENT(insertInputValue)) {
            fragment[insertInputKey] = insertInputValue;
            continue;
        }
        //Add id and type for referenced objects
        if (IS_APOLLO_OBJECT(insertInputValue)) {
            fragment[insertInputKey] = convertApolloObjectToRefObj(insertInputValue);
            continue;
        }
        //Add id and type for referenced objects arrays
        if (Array.isArray(insertInputValue)) {
            const innerArray = [];
            for (const arrayVal of insertInputValue) {
                if (IS_APOLLO_OBJECT(arrayVal)) {
                    innerArray.push(convertApolloObjectToRefObj(arrayVal));
                }
            }
            if (innerArray.length > 0) {
                fragment[insertInputKey] = innerArray;
            }
            continue;
        }
    }
    return fragment;
}
exports.convertInsertInputToShallowPartialFragment = convertInsertInputToShallowPartialFragment;
//# sourceMappingURL=utils.js.map